\name{read.affybatch.hybrid}
\alias{read.affybatch.hybrid}
\title{read celfiles from different chips into one AffyBatch object}
\description{
  This function reads celfile from different chips into one \code{AffyBatch} object, by converting coordinate of probes that have the same sequence in different chips. And a new cdf environement variable that would only use common probes would be assigned to global environment.
}
\usage{
read.affybatch.hybrid(..., filenames = character(0), phenoData = new("AnnotatedDataFrame"),
		description = NULL, notes = "", compress = getOption("BioC")$affy$compress.cel,
		rm.mask = FALSE, rm.outliers = FALSE, rm.extra = FALSE, verbose = FALSE,
		sd = FALSE, cdfname = NULL, cdfmapping = NULL, minpbstsize=3)
}
\arguments{
  \item{\dots}{ file names separated by comma.}
  \item{filenames}{ file names in a character vector.}
  \item{phenoData}{ a \code{\link[Biobase]{phenoData}} object}
  \item{description}{ a \code{\link[Biobase]{MIAME}} object }
  \item{notes}{ notes }
  \item{compress}{are the CEL files compressed ?}
  \item{rm.mask}{ should the spots marked as 'MASKS' set to \code{NA} ? }
  \item{rm.outliers}{ should the spots marked as 'OUTLIERS' set to \code{NA} }
  \item{rm.extra}{ if \code{TRUE}, overrides what is in \code{rm.mask} and \code{rm.oultiers} }
  \item{verbose}{ verbosity flag }
  \item{sd}{should the standard deviation values in the CEL file be read
    in. Since these are typically not used default is not to read them
    in. This also save lots of memory.}
   \item{cdfname}{Used to specify the name of an alternative cdf package of returned \code{AffyBatch} object. If set to \code{NULL}, the cdf package based on parameter cdfmapping will be used.}
  \item{cdfmapping}{Used to specify the mapping between original cdf name and an alternative cdf package of all chips of celfiles. If set to \code{NULL}, the usual cdf packages based on Affymetrix' mappings will be used.}
  \item{minpbstsize}{A minimum number of probes in a probeset. Probeset with a less number of probes would be removed. Default is 3, this would eliminate all probesets that have only 1-3  probes after removing probe.}
}
\details{
  \code{read.affybatch.hybrid} is an extended version of read.affybatch. If those celfiles are from the same chip, original function read.affybatch would be used. But if they are from different chips, All other chips would be converted to an alpha chip that either is specified by parameter cdfname or has the minimal probe number by converting coordinate of probes that have the same sequence in all chips.

  And a new cdf environement variable would be assigned to global environment. It is tailored from the alpha chip by keeping only the probes that exist in all chips. This new cdf environment would be used automatically whenever the returned \code{AffyBatch} object is analyzed.
}
\value{
  An \code{AffyBatch} object.
}
\author{Manhong Dai}
\examples{
	celpath <- paste(find.package("CustomCDF"),"celfiles",sep="/")
	fns <- list.celfiles(path=celpath,full.names=TRUE)
	map <- NULL
#if hs133ahsentrezg and hs133phsentrezg custom cdf R package are installed, uncomment following lines to use ENTREZG custom cdf
#	map <- c('HGU133A_Hs_ENTREZG', 'HGU133Plus2_Hs_ENTREZG')
#	names(map) <- c('HG-U133A', 'HG-U133_Plus_2')
	ab <- read.affybatch.hybrid(filenames=fns, cdfmapping=map)
	ls() ##A new cdf environment variable 'HG-U133A' would be generated
	eset <- rma(ab)
}
\keyword{manip}
