\name{removeprobe}
\alias{removeprobe}
\title{remove probe from cdf environment}
\description{
  This function removes unwanted probe from cdf environment
}
\usage{
removeprobe(object, xyMatrix=NULL, pbMatrix=NULL, minpbstsize=1, maxpbstsize=NULL, leftorright=c('left', 'right'))
}
\arguments{
  \item{object}{an \code{\link[affy:AffyBatch-class]{AffyBatch}}}
  \item{xyMatrix}{Object of class \code{"matrix"} containing coordinate of unwanted probes. The first column is X, the second column is Y. Both start from 0}
  \item{pbMatrix}{Object of class \code{"matrix"} containing probeset name and index of unwanted probes. The first column is probeset name, the second column is probe index in the probeset, starting from 1}
  \item{minpbstsize}{A minimum number of probes in a probeset. Probeset that has less probes would be removed. Default is 1.}
  \item{maxpbstsize}{A maximum number of probes in a probeset. In a probeset that has greater probes, only maxpbstsize of probes would be kept.}
  \item{leftorright}{Which side of probes would be kept in a probeset that has probes number greater than maxpbstsize.}
}
\details{
  The first step of this function is to remove probes that are in argument xyMatrix and pbMatrix, this step would be ignored if the two arguments are NULL.

  For probes in argument 'xyMatrix', this probe would be removed from all probesets that include the probe.

  For probes in argument 'pbMatrix', other probes that have the same coordinate but are in other probesets won't be removed. This is useful only when one wants to remove a probe from a specific probeset, and keeps it in other probesets.

  The second step is remove probesets that have probes less than argument 'minpbstsize'

  The third step is to keep one side of the probes. this step would be ignored if argument 'maxpbstsize' is NULL.
}
\value{
 a list of removed probe and probeset
}
\author{Manhong Dai}
\examples{
if (require(affydata)) {
  data(Dilution)
  ###############################################################
  #This would do a rma with both original cdf and modifed cdf, 
  #Compare the two txt files to see the difference
  ###############################################################
  esNormal <- rma(Dilution, normalize=FALSE, background=FALSE)
  write.exprs(esNormal, file='esNormal.txt')
  xy <- matrix(c(25,12, 1,2), ncol=2, dimnames = list(c(), c("X", "Y")))
  pb <- matrix(c('31307_at', '31307_at', '31308_at', 5, 4, 1),
	  ncol=2, dimnames = list(c(), c("probeset", "index")))
  report <- removeprobe(Dilution, xyMatrix=xy, pbMatrix=pb, minpbstsize=3);
  report
  esFilter <- rma(Dilution, normalize=FALSE, background=FALSE)
  write.exprs(esFilter, file='esFilter.txt')
  rm(list = c(Dilution@cdfName))

  ###############################################################
  #Each probeset in the following modified cdf would only include
  #3 probes from left side.
  ###############################################################
  report <- removeprobe(Dilution, maxpbstsize=3, leftorright='left');
}
}
\keyword{manip}
